; ---------------------------------------------------------------------------
; TileMinimizer - mirc v5.9x+ script to hide inactive windows
; v1.00.26
; 11/03 - 6/25/05, by Mouser
; ---------------------------------------------------------------------------
; This script is meant to automatically minimize mirc channel windows that
;  are inactive for some period of time, and pop them back up when someone
;  types on them (but not on other events).  This should allow you to idle
;  in some quiet channels without them using up your valuable screen realestate.
;
; IMPORTANT NOTE: It is *ONLY* useful when mirc is set to TILE mode - using it
;  in other modes is counterproductive and may cause confusion on your part.
;
; After loading you will have a new submenu for TileMinimizer in your main
;  Commands toolbar menu where you can change basic settings.
; And a new submenu when you right-click in a channel window, which will
;  allow you to exclude or stop excluding that window from minimization;
; ---------------------------------------------------------------------------



; ---------------------------------------------------------------------------
; initialization startup each time you launch mirc
on *:START: {
  ; version number
  %cam_scriptversion = 1.00.23

  ; load past window state data
  .cam_loadwindowstatedata

  ; let user know it's running
  echo 13 -si2 TileMinimzer script v $+ %cam_scriptversion loaded and ready.

  ; set up default variables if not already set
  .cam_setupvariables

  ; start the timer(s)
  .cam_starttimers
}

on 1:LOAD: {
  echo 7 -si2 TileMinimzer script is now installed.
}

on 1:UNLOAD: {
  .cam_savewindowstatedata
  .cam_stoptimers
}
; ---------------------------------------------------------------------------



; ---------------------------------------------------------------------------
alias cam_setupvariables {
  ; clear some temp variables
  .unset %cam_tempactivating

  ; defaults to every 10 minutes (setting this to x can actually lead to 2x minutes before closing)
  if (%cam_windowcheckinterval == $null) %cam_windowcheckinterval = 600
  ; if %cam_restoreactive == yes, then ALL windows not set specifically to -2 state will be restore on activity 
  if (%cam_restoreactive==$null) %cam_restoreactive = yes
  ; if %cam_restoreactive == notmanual, then only windows WE minimized will restore on activity 
  ;if (%cam_restoreactive==$null) %cam_restoreactive = notmanual

  ; the '%cam_dontreminimizebeforevisit yes' setting says that once a window pops up because someone types,
  ; it wont reminimize until the user actually visits it,
  ; which can be very useful if you are gone for a while and want to know which windows had activity
  if (%cam_dontreminimizebeforevisit == $null) %cam_dontreminimizebeforevisit = yes

  ; minimize mirc when all windows are minimized?
  if (%cam_minimizemirc == $null) %cam_minimizemirc = no

  ; should we leave the last window up and not minimize the last one
  if (%cam_leavelast == $null) %cam_leavelast = yes

  ; should a join event count as activity?
  if (%cam_wakeonjoin == $null) %cam_wakeonjoin = no
}

alias cam_starttimers {
  ; start a repeating timer for minimizing windows after delay
  .timercam_checkwindows 0 [ %cam_windowcheckinterval ] /cam_onwindowchecktimer

  ; a more frequent helper function to keep track of true minimization states
  ; attn: i dont think we really need this, as long as we run it when we disconnect
  ; .timercam_checkwindowstates 0 60 /cam_onwindowchecktimer_updatestates
}

alias cam_stoptimers {
  ; stop the timers
  .timercam_checkwindows off
  .timercam_checkwindowstates off
}

alias cam_delaytimers {
  ; this is a cute little routine called when user types something, to prevent a jarring trigger of window checking
  ; there are different ways we could do it, but we just do simplest, by reseting timer when they type anything
  .cam_stoptimers
  .cam_starttimers
}
; ---------------------------------------------------------------------------




; ---------------------------------------------------------------------------
menu menubar,status {
  &TileMinimizer
  .Modify settings:/camsettings
  .Minimize all windows:/cam_minimizeall
}


menu channel {
  &TileMinimizer
  .Minimize all windows:/cam_minimizeall
  .Exclude this channel from Tile Minimizer Processing:/cam_exludewindow $chan
  .Stop excluding this channel from Tile Minimizer Processing:/cam_stopexcludewindow $chan
}


menu query {
  &TileMinimizer
  .Minimize all windows:/cam_minimizeall
  .Exclude this query window from Tile Minimizer Processing:/cam_exludewindow $nick
  .Stop excluding this query window from Tile Minimizer Processing:/cam_stopexcludewindow $nick
}


alias camsettings {
  ; provide a somewhat nicer way for user to set values
  var %cam_windowcheckintervalNEW , %cam_restoreactiveNEW , %cam_dontreminimizebeforevisitNEW , %cam_minimizemircNEW , %cam_leavelastNEW
  %cam_windowcheckintervalNEW = $input(Number of seconds between checks for inactivity in a window:,1,TileMinimizer,%cam_windowcheckinterval)
  %cam_restoreactiveNEW = $input(Restore windows that become active? ('yes' = all windows , 'notmanual' = don't restore manually minimized windows): ,1,TileMinimizer,%cam_restoreactive)
  %cam_dontreminimizebeforevisitNEW = $input(Prevent re-minimization of windows until they are visited? ('yes' or 'no'): ,1,TileMinimizer,%cam_dontreminimizebeforevisit)
  %cam_minimizemircNEW = $input(Minimize mirc when all other windows are minimized? ('yes' or 'no'): ,1,TileMinimizer,%cam_minimizemirc )
  %cam_leavelastNEW = $input(Never minimize the last window? ('yes' or 'no'): ,1,TileMinimizer,%cam_leavelast)

  if (%cam_windowcheckintervalNEW != $null) %cam_windowcheckinterval = %cam_windowcheckintervalNEW
  if (%cam_restoreactiveNEW != $null) %cam_restoreactive = %cam_restoreactiveNEW 
  if (%cam_dontreminimizebeforevisitNEW != $null) %cam_dontreminimizebeforevisit = %cam_dontreminimizebeforevisitNEW 
  if (%cam_minimizemircNEW != $null) %cam_minimizemirc = %cam_minimizemircNEW 
  if (%cam_leavelastNEW != $null) %cam_leavelast = %cam_leavelastNEW 

  .cam_starttimers
}
; ---------------------------------------------------------------------------




; ---------------------------------------------------------------------------
; EVENTS

; incoming - incoming from channel
on *:text:*:#: {
  .cam_windowactivity $chan
}

; incoming - incoming from query window
on *:text:*:?: {
  .cam_windowactivity $nick
} 

; incoming - incoming from notice - seems to be overridden by text
on *:notice:*:*: { 
  .cam_windowactivity $chan
}

; incoming - incoming action from channel
on ^*:action:*:#: {
  .cam_windowactivity $chan
}

; incoming - incoming action from query window
on ^*:action:*:?: {
  .cam_windowactivity $nick
}

; incoming - incoming from dcc chat
on *:chat:*: {
  var %cname = $chr(61) $+ $nick
  .cam_windowactivity %cname
}

; user activates a window (so even without typing, merely viewing will srest usage count)
on *:active:*: {
  ;  /echo @DEBUG active is $active
  ; we return if we are in the process of minimizing lots of windows, because active windows may change
  if (%cam_tempactivating == yes) return
  var %cname = $active
  if (%cname != $null) {
    if ($cam_iswindowminimized(%cname) == $false) {
      ;      /echo @DEBUG active is $active and we are visiting it
      .cam_delaytimers
      .cam_windowactivity %cname
      .cam_windowactivity_visit %cname
    }
  }
}

; outgoing - outgoing to channel
on *:input:*: {
  var %cname = $active
  .cam_delaytimers
  .cam_windowactivity %cname
  .cam_windowactivity_visit %cname
}

; triggered on connect
on *:CONNECT: {
  ; reminimize any windows that should be minimized
  .cam_reminimizeonconnect
}

; disconnect
on *:DISCONNECT: {
  ; force it to update minimization window states
  .cam_onwindowchecktimer_updatestates
}


; windows open
on *:OPEN:?: {
  .cam_newwindow $nick
}
on *:OPEN:=: {
  var %cname
  %cname = = $+ $nick
  .cam_newwindow %cname
}

on *:JOIN:#: {
  .cam_newwindow $chan
  if (%cam_wakeonjoin == yes) .cam_windowactivity $chan

  ; we can specify certain channels to restore on join
  if ($chan isin %cam_joinchans) {
    ; play sound
    if (%cam_joinchansbeep > 0) .beep %cam_joinchansbeep
    ; show window
    if (%cam_minimizemirc == yes) .cam_restoremirc
    ; restore win
    /window -r $chan
    ; update the data for this window to say it has some activity on it
    if ($cam_getwindowchecktag($chan) != -9) .cam_setwindowchecktag $chan 0
  }

}

; windows close
on *:CLOSE:?: {
  .cam_closewindow $nick
}
on *:CLOSE:=: {
  var %cname
  %cname = = $+ $nick
  .cam_closewindow %cname
}
on *:PART:#: {
  .cam_closewindow $chan
}


on *:SIGNAL:CAM_WindowActivity: {
  ; this is a signal that people can use in their code to wake up a window when they output some text to it
  ; this might be nescesary because this script can only detect incoming text and user-typed text
  ;  not text generated by some script on your machine.
  ; generate the signal like: /signal WindowActivity $chan
  .cam_windowactivity $1
}
; ---------------------------------------------------------------------------


; ---------------------------------------------------------------------------
alias cam_newwindow {
  ;  var %cname = $1
  ;  if (%cname == $null) return
  ;
  ;  echo DEBUG newwindow cname is %cname connecting is %cam_connecting tag is $cam_getwindowchecktag(%cname)
  ;  if (%cam_connecting == yes) {
  ;    ; ok, here is our interesting check.  if windows open right after connecting, then its mirc or bnc auto-opening them
  ;    ;  so, if the channel was minimized on last run, we reminimize it
  ;    ;  or, if not, we reset usage inactivity tag
  ;    if ( ($cam_getwindowchecktag(%cname) == -1) || ($cam_getwindowchecktag(%cname) == -2) ) {
  ;      ; this was minimized last time, so we reminmize
  ;      .cam_minimizewindow %cname
  ;      return
  ;    }
  ;  }
  ;  ; unless the window is set to be ignored, reset its counter
  ;  if ($cam_getwindowchecktag(%cname) != -9) .cam_setwindowchecktag %cname 0
}


alias cam_closewindow {
  ; do nothing because we want to remember its state
  ;
  ;  var %cname = $1
  ;  if (%cname == $null) return
  ;  ; unless the window is set to be ignored, reset its counter
  ;  if ($cam_getwindowchecktag(%cname) != -9) .cam_setwindowchecktag %cname 0
}
; ---------------------------------------------------------------------------


; ---------------------------------------------------------------------------
alias cam_reminimizeonconnect {
  ; we have just reconnected, and bnc may have auto-opened windows we want to re-minimize
  var %ccount = 1
  var %cname

  ;set flag to prevent change in focus fooling us into thinking user has activated windows
  %cam_tempactivating = yes

  ;echo DEBUG reminimizing loop
  while ($window(*,%ccount) != $null) {
    %cname = $window(*,%ccount)
    if ( ($cam_getwindowchecktag(%cname) == -1) || ($cam_getwindowchecktag(%cname) == -2) ) {
      ; this window SHOULD be minimized
      if ($cam_iswindowminimized(%cname) == $false) {
        ; its not so force it
        ;        echo @DEBUG forcing min of %cname
        .cam_minimizewindow %cname
      }
      ; normalize all others to 1
      else if ($cam_getwindowchecktag(%cname) > 1) {
        .cam_setwindowchecktag %cname 1
        if (%cname == $null) /echo DEBUG1 null
      }
    }
    inc %ccount
  }

  .timercam_unsettempactivating 1 1 .unset %cam_tempactivating
}
; ---------------------------------------------------------------------------













; ---------------------------------------------------------------------------
alias cam_windowactivity {
  ; some text activity has occured on a channel, so we want to check the channel
  ; and update its time-of-last-activity
  ; and unminimize the window iff WE were the one who minimized it
  ; $1 is the window name
  var %cname = $1

  ; correct for bad channel names
  if (%cname == $null) %cname = $active
  if (%cname == Status Window) %cname = Status
  if ($cam_getwindowchecktag(%cname) == -9) return

  ;set flag to prevent change in focus fooling us into thinking user has activated windows
  %cam_tempactivating = yes

  ;  /echo @DEBUG activity on window %cname

  ; now if it is minimized AND we were the one to minimize it,
  ; AND we are set to restore-on-activity, then restore it
  if ( $cam_iswindowminimized(%cname) ) {
    ; yes the window is minimized, BUT does it matter if it was us that minimized or did the user do it manually?
    ;/echo 4 DEBUG win is minimized
    if ( (%cam_restoreactive == yes) || ( (%cam_restoreactive == notmanual) && ($cam_getwindowchecktag_didweminimize(%cname) == $true) ) ) {
      ;/echo 4 DEBUG and we will try to restore it
      ; ok, either we minimized it or settings say it don't matter
      ; save active window
      var %activechan = $active
      if (%activechan == %cname) %activechan = $null
      if ( (%activechan != $null) && ($cam_iswindowminimized(%activechan)) ) %activechan = $null
      if (%activechan == Status) %activechan = "Status Window"
      if (%activechan == Status Window) %activechan = "Status Window"
      ;restore minimized window
      ;      /echo @DEBUG trying to restore %cname
      .cam_restorewindow %cname
      ; restoring a window will switch to it, which would be very disconcerting if we dont switch back
      if (%activechan != $null) {
        ;        /echo @DEBUG trying to reactivate %activechan
        /window -a %activechan
      }
    }
    else {
      ;/echo 4 DEBUG but window is not minimized
    }
  }

  ; update the data for this window to say it has some activity on it
  .cam_updatewindowchecktagonusage %cname

  .timercam_unsettempactivating 1 1 .unset %cam_tempactivating

  if ($cam_getwindowchecktag($1) == -9) return
  if (%cam_minimizemirc == yes) .cam_restoremirc
}


alias cam_windowactivity_visit {
  ; this is called when user actually VISITS a window, which is used in conjunction with the '%cam_dontreminimizebeforevisit yes' setting
  ; which says that once a window pops up because someone types, it wont reminimize until the user actually visits it,
  ; which can be very useful if you are gone for a while and want to know which windows had activity
  if ($cam_getwindowchecktag($1) != -9) {
    .cam_setwindowchecktag $1 1
    if ($1 == $null) /echo DEBUG2 null
  }
}


alias cam_restorewindow {
  ; restore the window $1
  var %cname = $1

  if (%cname == Status) %cname = "Status Window"
  if (%cname == Status Window) %cname = "Status Window"

  if ($cam_getwindowchecktag($1) == -9) return

  ; restore mirc if its hidden
  if (%cam_minimizemirc == yes) .cam_restoremirc
  ; restore win
  ; update the data for this window to say it has some activity on it
  ;  /echo @DEBUG restoring win %cname
  /window -r %cname
  .cam_setwindowchecktag $1 0
}


alias cam_iswindowminimized {
  ; return true if the winow is minimized
  if ($window($1,1).state == minimized) || ($window($1,1).state == hidden) return $true
  return $false
}
; ---------------------------------------------------------------------------




; ---------------------------------------------------------------------------
alias cam_onwindowchecktimer {
  ; some time has passed, so check which windows need to be hidden
  ; we can use style1 check or style2 check (not both!)

  ;set flag to prevent change in focus fooling us into thinking user has activated windows
  %cam_tempactivating = yes

  ; check all windows
  /cam_onwindowchecktimer_style2

  .timercam_unsettempactivating 1 1 .unset %cam_tempactivating
}


alias cam_onwindowchecktimer_style2 {
  ; this style checks by looping through windows
  var %ccount
  var %nonminimized
  var %cname
  var %activewin = $null

  ; first just count how many windows are NON-minimized
  %nonminimized = 0
  %ccount = 1
  while ($window(*,%ccount) != $null) {
    %cname = $window(*,%ccount)
    if ( $cam_iswindowminimized(%cname) == $false ) {
      %nonminimized = %nonminimized + 1
    }
    inc %ccount
  }

  ; if there is noting to minimize we are done
  if (%nonminimized == 0) return
  if ((%nonminimized == 1) && (%cam_leavelast == yes) ) return

  ; now we minimize them
  %ccount = 1
  while ($window(*,%ccount) != $null) {
    %cname = $window(*,%ccount)
    if ( $cam_iswindowminimized(%cname) == $false ) {
      if (%cname == $active) {
        ; we save activewin so we can check it AFTER all others
        %activewin = %cname
      }
      else {
        .cam_checkwindow %cname
        if ( $cam_iswindowminimized(%cname) == $true ) {
          %nonminimized = %nonminimized - 1
          if ((%nonminimized == 1) && (%cam_leavelast == yes) ) return
        }
      }
    }
    else {
      ; before we return, make sure if user minimized this, that we set usage count to 0
      ; if the count >= 0 then it means that the user minimized it by herself, so we record that fact with a -2
      if ($cam_getwindowchecktag(%cname) >= 0) {
        ;/echo 4 DEBUG forcving code to -2
        .cam_setwindowchecktag %cname -2
      }
    }
    inc %ccount
  }

  ; if there is noting to minimize we are done
  if ((%nonminimized == 1) && (%cam_leavelast == yes) ) return

  ; now check active window - we do this last SO THAT if user has told us to keep one window open, it is always the active one
  if (%activewin != $null) {
    ; now do active win
    %cname = %activewin
    .cam_checkwindow %cname
    if ( $cam_iswindowminimized(%cname) == $true ) {
      %nonminimized = %nonminimized - 1
    }
  }

  ; minimize mirc if we just minimized last window
  if ( (%nonminimized == 0 ) && (%cam_minimizemirc == yes) ) .cam_hidemirc
}



alias cam_checkwindow {
  ; some time has passed, check window $1 to see if its time to minimize it
  var %windowchecktag
  var %cname = $1

  ;/echo 4 DEBUG checking status of window $1

  ; if it's already minimized then return
  if ( $cam_iswindowminimized(%cname) ) {
    ;/echo 4 DEBUG win is already minimized
    ; before we return, make sure if user minimized this, that we set usage count to 0
    ; if the count >= 0 then it means that the user minimized it by herself, so we record that fact with a -2
    if ($cam_getwindowchecktag(%cname) >= 0) {
      ;/echo 4 DEBUG forcving code to -2
      .cam_setwindowchecktag %cname -2
    }
    return
  }

  ; otherwise increment NON-usage counter (2 nonusages in a row will trigger minimization)
  .cam_updatewindowchecktagonnonusage %cname

  ; ok it's not already minmized, now should we minimize it?
  %windowchecktag = $cam_getwindowchecktag(%cname)

  ;/echo 4 DEBUG window tag = %windowchecktag

  ; if %windowchecktag hits 3 then it means there was no activity since our last check
  if (%windowchecktag == 3) {
    ; ok we want to minimize it
    .cam_minimizewindow %cname
  }
}


alias cam_minimizewindow {
  ; minimize window $1
  var %cname = $1

  if (%cname == Status) %cname = "Status Window"
  if (%cname == Status Window) %cname = "Status Window"
  ;  /echo @DEBUG minimizig %cname
  if (%cname != $null) /window -n %cname
  ; and reset check tag to say that we have minimized it
  .cam_setwindowchecktag_weminimized $1
}

alias cam_updatewindowchecktagonusage {
  ; updatet the tag for window $1
  ; and return the value after incrementing
  ; some tags may be set to -9 which is a way of saying IGNORE this window, so we dont modify those
  var %currenttag = $cam_getwindowchecktag($1)

  ; ignore if its set to ignore
  if (%currenttag == -9) return
  ; if we have just restored it, then we leave count at 0 so we can know not to reminimize itafter some delay until they visit it
  if (%currenttag == 0) return
  ;set it
  .cam_setwindowchecktag $1 1
  if ($1 == $null) /echo DEBUG3 null

  ; return it
  return %currenttag
}


alias cam_updatewindowchecktagonnonusage {
  ; updatet the tag for window $1
  ; and return the value after incrementing
  ; some tags may be set to -9 which is a way of saying IGNORE this window, so we dont modify those
  ; note this is only called if the window is NOT minimized
  var %currenttag = $cam_getwindowchecktag($1)

  ; update
  if (%currenttag == $null) %currenttag = 1
  else if (%currenttag == -9) return
  else if (%currenttag == -2) %currenttag = 1
  else if (%currenttag == -1) %currenttag = 1

  ; when set to 0 we WAIT until user visits the chan or types in it, and we do NOT update counter and reminimize
  else if ( (%currenttag == 0) && (%cam_dontreminimizebeforevisit == yes) ) %currenttag = 0

  else if (%currenttag == 0) %currenttag = 2
  else if (%currenttag == 1) %currenttag = 2
  else if (%currenttag == 2) %currenttag = 3

  ;set it
  .cam_setwindowchecktag $1 %currenttag
  if ($1 == $null) /echo DEBUG4 null

  ; return it
  return %currenttag
}



alias cam_setwindowchecktag_weminimized {
  ; set tag to say we minimized window $1
  if ($cam_getwindowchecktag($1) != -9) {
    .cam_setwindowchecktag $1 -1
  }
}


alias cam_getwindowchecktag_didweminimize {
  ; return $true if we minimized window $1
  if ($cam_getwindowchecktag($1) == -1) return $true
  return $false
}
; ---------------------------------------------------------------------------


; ---------------------------------------------------------------------------
alias cam_onwindowchecktimer_updatestates {
  ; this style checks by looping through windows
  var %ccount = 1

  while ($window(*,%ccount) != $null) {
    .cam_checkwindow_forminstate $window(*,%ccount)
    inc %ccount
  }
}

alias cam_checkwindow_forminstate {
  ; some time has passed, check window $1 to update its true minimization state
  ; if it's already minimized then return
  if ( $cam_iswindowminimized($1) ) {
    ; before we return, make sure if user minimized this, that we set usage count to 0
    ; if the count >= 0 then it means that the user minimized it by herself, so we record that fact with a -2
    if ($cam_getwindowchecktag($1) >= 0) .cam_setwindowchecktag $1 -2
  }
}
; ---------------------------------------------------------------------------


; ---------------------------------------------------------------------------
alias cam_getwindowchecktag {
  ; return the usage tag for window $1
  ; we will use a hash for this eventually but for testing we use variables
  var %usetag = [ % $+ [ cam_windousetag_ $+ [ $1 ]  ] ]
  return %usetag
}

alias cam_setwindowchecktag {
  ; set window check flag for window $1 to $2
  set [ % $+ [ cam_windousetag_ $+ [ $1 ]  ] ] $2
  ;/echo 4 DEBUG set window check tag for $1 to $2  
}
; ---------------------------------------------------------------------------



; ---------------------------------------------------------------------------
alias cam_hidemirc {
  ; all windows are minimized, so minimize mirc itself if appropriate
  ;echo DEBUG hiding mirc
  if ($window(-2).state != minimized) {
    ;echo DEBUG minimizing mirc
    /showmirc -n
  }
}


alias cam_restoremirc {
  ; restore mirc iff it is hidden
  if ($window(-2).state == minimized) {
    ;echo DEBUG restoring mirc
    /showmirc -r
  }
}
; ---------------------------------------------------------------------------





; ---------------------------------------------------------------------------
alias cam_restorestatesonconnect {
  ; this is called after we connect to a server, and after we have loaded our settings
  ; make sure windows are in the state we left them last time we were in mirc, called on connect
  ; for this procedure, we simply walk through our windows, and any with tags set to -1 (means we minimized it last time we checked)
  ; we will reminimize
}

alias cam_loadwindowstatedata {
  ; load window state data from file
}

alias cam_savewindowstatedata {
  ; save window state data to file
}
; ---------------------------------------------------------------------------



; ---------------------------------------------------------------------------
alias cam_minimizeall {
  ; minimize all windows and mark them as visited
  var %ccount
  var %cname

  ; first just count how many windows are NON-minimized
  %ccount = 1
  while ($window(*,%ccount) != $null) {
    %cname = $window(*,%ccount)
    .cam_minimizewindow %cname
    inc %ccount
  }
}
; ---------------------------------------------------------------------------


; ---------------------------------------------------------------------------
alias cam_exludewindow {
  ; $1 is channel or nick name
  .cam_setwindowchecktag $1 -9
  echo 4 Window $1 is now excluded from processing by TileMinimizer (will not be minimized or restored automatically)
}

alias cam_stopexcludewindow {
  ; $1 is channel or nick name
  .cam_setwindowchecktag $1 1
  echo 4 Window $1 is now enabled for processing by TileMinimizer (will be minimized on inactivity, and restored on activity)
}
; ---------------------------------------------------------------------------
